@@ -72,6 +72,11 @@ class QgsLabelingEngineSettings;
 class QgsAuxiliaryStorage;
 class QgsMapLayer;
 class QgsBookmarkManager;
+class QgsProjectViewSettings;
+class QgsProjectDisplaySettings;
+class QgsProjectTimeSettings;
+class QgsAnnotationLayer;
+
 
 /**
  * \ingroup core
@@ -98,15 +103,79 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     Q_PROPERTY( QgsMapThemeCollection *mapThemeCollection READ mapThemeCollection NOTIFY mapThemeCollectionChanged )
     Q_PROPERTY( QgsSnappingConfig snappingConfig READ snappingConfig WRITE setSnappingConfig NOTIFY snappingConfigChanged )
     Q_PROPERTY( QgsRelationManager *relationManager READ relationManager )
+    Q_PROPERTY( AvoidIntersectionsMode avoidIntersectionsMode READ avoidIntersectionsMode WRITE setAvoidIntersectionsMode NOTIFY avoidIntersectionsModeChanged )
     Q_PROPERTY( QList<QgsVectorLayer *> avoidIntersectionsLayers READ avoidIntersectionsLayers WRITE setAvoidIntersectionsLayers NOTIFY avoidIntersectionsLayersChanged )
     Q_PROPERTY( QgsProjectMetadata metadata READ metadata WRITE setMetadata NOTIFY metadataChanged )
     Q_PROPERTY( QColor backgroundColor READ backgroundColor WRITE setBackgroundColor NOTIFY backgroundColorChanged )
     Q_PROPERTY( QColor selectionColor READ selectionColor WRITE setSelectionColor NOTIFY selectionColorChanged )
+    Q_PROPERTY( bool topologicalEditing READ topologicalEditing WRITE setTopologicalEditing NOTIFY topologicalEditingChanged )
 
   public:
+
+    /**
+     * Flags which control project read behavior.
+     * \since QGIS 3.10
+     */
+    enum class ReadFlag SIP_MONKEYPATCH_SCOPEENUM
+    {
+      FlagDontResolveLayers = 1 << 0, //!< Don't resolve layer paths (i.e. don't load any layer content). Dramatically improves project read time if the actual data from the layers is not required.
+      FlagDontLoadLayouts = 1 << 1, //!< Don't load print layouts. Improves project read time if layouts are not required, and allows projects to be safely read in background threads (since print layouts are not thread safe).
+      FlagTrustLayerMetadata = 1 << 2, //!< Trust layer metadata. Improves project read time. Do not use it if layers' extent is not fixed during the project's use by QGIS and QGIS Server.
+    };
+    Q_DECLARE_FLAGS( ReadFlags, ReadFlag )
+
+    /**
+     * Flags which control project read behavior.
+     * \since QGIS 3.12
+     */
+    enum class FileFormat
+    {
+      Qgz, //!< Archive file format, supports auxiliary data
+      Qgs, //!< Project saved in a clear text, does not support auxiliary data
+    };
+    Q_ENUM( FileFormat )
+
+    /**
+     * Flags which control how intersections of pre-existing feature are handled when digitizing new features.
+     * \since QGIS 3.14
+     */
+    enum class AvoidIntersectionsMode
+    {
+      AllowIntersections, //!< Overlap with any feature allowed when digitizing new features
+      AvoidIntersectionsCurrentLayer, //!< Overlap with features from the active layer when digitizing new features not allowed
+      AvoidIntersectionsLayers, //!< Overlap with features from a specified list of layers when digitizing new features not allowed
+    };
+    Q_ENUM( AvoidIntersectionsMode )
+
+    /**
+     * Data defined properties.
+     * Overrides of user defined server parameters are stored in a
+     * property collection and they can be retrieved using the
+     * indexes specified in this enum.
+     *
+     * \since QGIS 3.14
+     */
+    enum DataDefinedServerProperty
+    {
+      NoProperty = 0, //!< No property
+      AllProperties = 1, //!< All properties for item
+      WMSOnlineResource = 2, //!< Alias
+    };
+
     //! Returns the QgsProject singleton instance
     static QgsProject *instance();
 
+    /**
+     * Set the current project singleton instance to \a project
+     *
+     * \note this method is provided mainly for the server, which caches the projects and (potentially) needs to switch the current instance on every request.
+     * \warning calling this method can have serious, unintended consequences, including instability, data loss and undefined behavior. Use with EXTREME caution!
+     * \see instance()
+     * \since QGIS 3.10.11
+     */
+    static void setInstance( QgsProject *project ) ;
+
+
     /**
      * Create a new QgsProject.
      *
@@ -135,6 +204,38 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     */
     QString title() const;
 
+    /**
+    * Returns the user name that did the last save.
+    *
+    * \see saveUserFullName()
+    *
+    * \since QGIS 3.12
+    */
+    QString saveUser() const;
+
+    /**
+    * Returns the full user name that did the last save.
+    *
+    * \see saveUser()
+    *
+    * \since QGIS 3.12
+    */
+    QString saveUserFullName() const;
+
+    /**
+     * Returns the date and time when the project was last saved.
+     *
+     * \since QGIS 3.14
+     */
+    QDateTime lastSaveDateTime() const;
+
+    /**
+     * Returns the QGIS version which the project was last saved using.
+     *
+     * \since QGIS 3.14
+     */
+    QgsProjectVersion lastSaveVersion() const;
+
     /**
      * Returns TRUE if the project has been modified since the last write()
      */
@@ -156,6 +257,32 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     */
     QString fileName() const;
 
+    /**
+     * Sets the original \a path associated with the project.
+     *
+     * This is intended for use with non-qgs/qgz project files (see QgsCustomProjectOpenHandler) in order to allow
+     * custom project open handlers to specify the original file name of the project. For custom project formats,
+     * it is NOT appropriate to call setFileName() with the original project path, as this causes the original (non
+     * QGIS) project file to be overwritten when the project is next saved.
+     *
+     * \see originalPath()
+     * \since QGIS 3.14
+     */
+    void setOriginalPath( const QString &path );
+
+    /**
+     * Returns the original path associated with the project.
+     *
+     * This is intended for use with non-qgs/qgz project files (see QgsCustomProjectOpenHandler) in order to allow
+     * custom project open handlers to specify the original file name of the project. For custom project formats,
+     * it is NOT appropriate to call setFileName() with the original project path, as this causes the original (non
+     * QGIS) project file to be overwritten when the project is next saved.
+
+     * \see setOriginalPath()
+     * \since QGIS 3.14
+    */
+    QString originalPath() const;
+
     /**
      * Returns QFileInfo object for the project's associated file.
      *
@@ -267,24 +394,13 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     void clear();
 
-    /**
-     * Flags which control project read behavior.
-     * \since QGIS 3.10
-     */
-    enum ReadFlag
-    {
-      FlagDontResolveLayers = 1 << 0, //!< Don't resolve layer paths (i.e. don't load any layer content). Dramatically improves project read time if the actual data from the layers is not required.
-      FlagDontLoadLayouts = 1 << 1, //!< Don't load print layouts. Improves project read time if layouts are not required, and allows projects to be safely read in background threads (since print layouts are not thread safe).
-    };
-    Q_DECLARE_FLAGS( ReadFlags, ReadFlag )
-
     /**
      * Reads given project file from the given file.
      * \param filename name of project file to read
      * \param flags optional flags which control the read behavior of projects
      * \returns TRUE if project file has been read successfully
      */
-    bool read( const QString &filename, QgsProject::ReadFlags flags = nullptr );
+    bool read( const QString &filename, QgsProject::ReadFlags flags = QgsProject::ReadFlags() );
 
     /**
      * Reads the project from its currently associated file (see fileName() ).
@@ -294,7 +410,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      *
      * \returns TRUE if project file has been read successfully
      */
-    bool read( QgsProject::ReadFlags flags = nullptr );
+    bool read( QgsProject::ReadFlags flags = QgsProject::ReadFlags() );
 
     /**
      * Reads the layer described in the associated DOM node.
@@ -325,95 +441,158 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     bool write();
 
     /**
-     * Write a boolean entry to the project file.
+     * Write a boolean \a value to the project file.
      *
      * Keys are '/'-delimited entries, implying
      * a hierarchy of keys and corresponding values
      *
      * \note The key string must be valid xml tag names in order to be saved to the file.
      * \note available in Python bindings as writeEntryBool
+     *
+     * \see readBoolEntry()
      */
     bool writeEntry( const QString &scope, const QString &key, bool value ) SIP_PYNAME( writeEntryBool );
 
     /**
-     * Write a double entry to the project file.
+     * Write a double \a value to the project file.
      *
      * Keys are '/'-delimited entries, implying
      * a hierarchy of keys and corresponding values
      *
      * \note The key string must be valid xml tag names in order to be saved to the file.
      * \note available in Python bindings as writeEntryDouble
+     *
+     * \see readDoubleEntry()
      */
     bool writeEntry( const QString &scope, const QString &key, double value ) SIP_PYNAME( writeEntryDouble );
 
     /**
-     * Write an integer entry to the project file.
+     * Write an integer \a value to the project file.
      *
      * Keys are '/'-delimited entries, implying
      * a hierarchy of keys and corresponding values
      *
      * \note The key string must be valid xml tag names in order to be saved to the file.
+     *
+     * \see readNumEntry()
      */
     bool writeEntry( const QString &scope, const QString &key, int value );
 
     /**
-     * Write a string entry to the project file.
+     * Write a string \a value to the project file.
      *
      * Keys are '/'-delimited entries, implying
      * a hierarchy of keys and corresponding values
      *
      * \note The key string must be valid xml tag names in order to be saved to the file.
+     *
+     * \see readEntry()
      */
     bool writeEntry( const QString &scope, const QString &key, const QString &value );
 
     /**
-     * Write a string list entry to the project file.
+     * Write a string list \a value to the project file.
      *
      * Keys are '/'-delimited entries, implying
      * a hierarchy of keys and corresponding values
      *
      * \note The key string must be valid xml tag names in order to be saved to the file.
+     *
+     * \see readListEntry()
      */
     bool writeEntry( const QString &scope, const QString &key, const QStringList &value );
 
     /**
-     * Key value accessors
+     * Reads a string list from the specified \a scope and \a key.
      *
-     * keys would be the familiar QgsSettings-like '/' delimited entries,
-     * implying a hierarchy of keys and corresponding values
+     * \param scope entry scope (group) name
+     * \param key entry key name. Keys are '/'-delimited entries, implying a hierarchy of keys and corresponding values.
+     * \param def default value to return if the specified \a key does not exist within the \a scope.
+     * \param ok set to TRUE if key exists and has been successfully retrieved as a string list
+     *
+     * \returns entry value as a string list
      */
-    QStringList readListEntry( const QString &scope, const QString &key, const QStringList &def = QStringList(), bool *ok = nullptr ) const;
-
-    QString readEntry( const QString &scope, const QString &key, const QString &def = QString(), bool *ok = nullptr ) const;
-    int readNumEntry( const QString &scope, const QString &key, int def = 0, bool *ok = nullptr ) const;
-    double readDoubleEntry( const QString &scope, const QString &key, double def = 0, bool *ok = nullptr ) const;
-    bool readBoolEntry( const QString &scope, const QString &key, bool def = false, bool *ok = nullptr ) const;
-
-
-    //! Remove the given key
-    bool removeEntry( const QString &scope, const QString &key );
-
+    QStringList readListEntry( const QString &scope, const QString &key, const QStringList &def = QStringList(), bool *ok SIP_OUT = nullptr ) const;
 
     /**
-     * Returns keys with values -- do not return keys that contain other keys
+     * Reads a string from the specified \a scope and \a key.
+     *
+     * \param scope entry scope (group) name
+     * \param key entry key name. Keys are '/'-delimited entries, implying a hierarchy of keys and corresponding values.
+     * \param def default value to return if the specified \a key does not exist within the \a scope.
+     * \param ok set to TRUE if key exists and has been successfully retrieved as a string value
+     *
+     * \returns entry value as string from \a scope given its \a key
+     */
+    QString readEntry( const QString &scope, const QString &key, const QString &def = QString(), bool *ok SIP_OUT = nullptr ) const;
+
+    /**
+     * Reads an integer from the specified \a scope and \a key.
+     *
+     * \param scope entry scope (group) name
+     * \param key entry key name. Keys are '/'-delimited entries, implying a hierarchy of keys and corresponding values.
+     * \param def default value to return if the specified \a key does not exist within the \a scope.
+     * \param ok set to TRUE if key exists and has been successfully retrieved as an integer
+     *
+     * \returns entry value as integer from \a scope given its \a key
+     */
+    int readNumEntry( const QString &scope, const QString &key, int def = 0, bool *ok SIP_OUT = nullptr ) const;
+
+    /**
+     * Reads a double from the specified \a scope and \a key.
+     *
+     * \param scope entry scope (group) name
+     * \param key entry key name. Keys are '/'-delimited entries, implying a hierarchy of keys and corresponding values.
+     * \param default value to return if the specified \a key does not exist within the \a scope.
+     * \param ok set to TRUE if key exists and has been successfully retrieved as a double
+     *
+     * \returns entry value as double from \a scope given its \a key
+     */
+    double readDoubleEntry( const QString &scope, const QString &key, double def = 0, bool *ok SIP_OUT = nullptr ) const;
+
+    /**
+     * Reads a boolean from the specified \a scope and \a key.
+     *
+     * \param scope entry scope (group) name
+     * \param key entry key name. Keys are '/'-delimited entries, implying a hierarchy of keys and corresponding values.
+     * \param default value to return if the specified \a key does not exist within the \a scope.
+     * \param ok set to TRUE if key exists and has been successfully retrieved as a boolean
+     *
+     * \returns entry value as boolean from \a scope given its \a key
+     */
+    bool readBoolEntry( const QString &scope, const QString &key, bool def = false, bool *ok SIP_OUT = nullptr ) const;
+
+    /**
+     * Remove the given \a key from the specified \a scope.
+     */
+    bool removeEntry( const QString &scope, const QString &key );
+
+    /**
+     * Returns a list of child keys with values which exist within the the specified \a scope and \a key.
+     *
+     * This method does not return keys that contain other keys. See subkeyList() to retrieve keys
+     * which contain other keys.
      *
      * \note equivalent to QgsSettings entryList()
      */
     QStringList entryList( const QString &scope, const QString &key ) const;
 
     /**
-     * Returns keys with keys -- do not return keys that contain only values
+     * Returns a list of child keys which contain other keys that exist within the the specified \a scope and \a key.
+     *
+     * This method only returns keys with keys, it will not return keys that contain only values. See
+     * entryList() to retrieve keys with values.
      *
      * \note equivalent to QgsSettings subkeyList()
      */
     QStringList subkeyList( const QString &scope, const QString &key ) const;
 
+    // TODO Now slightly broken since re-factoring.  Won't print out top-level key
+    //           and redundantly prints sub-keys.
 
     /**
      * Dump out current project properties to stderr
      */
-    // TODO Now slightly broken since re-factoring.  Won't print out top-level key
-    //           and redundantly prints sub-keys.
     void dumpProperties() const;
 
     /**
@@ -430,7 +609,9 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     */
     QString writePath( const QString &filename ) const;
 
-    //! Turn filename read from the project file to an absolute path
+    /**
+     * Transforms a \a filename read from the project file to an absolute path.
+     */
     QString readPath( const QString &filename ) const;
 
     //! Returns error message from previous read/write
@@ -442,7 +623,11 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     void setBadLayerHandler( QgsProjectBadLayerHandler *handler SIP_TRANSFER );
 
-    //! Returns project file path if layer is embedded from other project file. Returns empty string if layer is not embedded
+    /**
+     * Returns the source project file path if the layer with matching \a id is embedded from other project file.
+     *
+     * Returns an empty string if the matching layer is not embedded.
+     */
     QString layerIsEmbedded( const QString &id ) const;
 
     /**
@@ -454,7 +639,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * \note not available in Python bindings
      */
     bool createEmbeddedLayer( const QString &layerId, const QString &projectFilePath, QList<QDomNode> &brokenNodes,
-                              bool saveFlag = true, QgsProject::ReadFlags flags = nullptr ) SIP_SKIP;
+                              bool saveFlag = true, QgsProject::ReadFlags flags = QgsProject::ReadFlags() ) SIP_SKIP;
 
     /**
      * Create layer group instance defined in an arbitrary project file.
@@ -463,7 +648,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      *
      * \since QGIS 2.4
      */
-    QgsLayerTreeGroup *createEmbeddedGroup( const QString &groupName, const QString &projectFilePath, const QStringList &invisibleLayers, QgsProject::ReadFlags flags = nullptr );
+    QgsLayerTreeGroup *createEmbeddedGroup( const QString &groupName, const QString &projectFilePath, const QStringList &invisibleLayers, QgsProject::ReadFlags flags = QgsProject::ReadFlags() );
 
     //! Convenience function to set topological editing
     void setTopologicalEditing( bool enabled );
@@ -533,7 +718,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     QgsRelationManager *relationManager() const;
 
     /**
-     * Returns the project's layout manager, which manages compositions within
+     * Returns the project's layout manager, which manages print layouts, atlases and reports within
      * the project.
      * \note not available in Python bindings
      * \since QGIS 3.0
@@ -541,7 +726,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     const QgsLayoutManager *layoutManager() const SIP_SKIP;
 
     /**
-     * Returns the project's layout manager, which manages compositions within
+     * Returns the project's layout manager, which manages print layouts, atlases and reports within
      * the project.
      * \since QGIS 3.0
      */
@@ -562,6 +747,55 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     QgsBookmarkManager *bookmarkManager();
 
+    /**
+     * Returns the project's view settings, which contains settings and properties
+     * relating to how a QgsProject should be viewed and behave inside a map canvas
+     * (e.g. map scales and default view extent)
+     * \note not available in Python bindings
+     * \since QGIS 3.10.1
+     */
+    const QgsProjectViewSettings *viewSettings() const SIP_SKIP;
+
+    /**
+     * Returns the project's view settings, which contains settings and properties
+     * relating to how a QgsProject should be viewed and behave inside a map canvas
+     * (e.g. map scales and default view extent)
+     * \since QGIS 3.10.1
+     */
+    QgsProjectViewSettings *viewSettings();
+
+    /**
+     * Returns the project's time settings, which contains the project's temporal range and other
+     * time based settings.
+     *
+     * \note not available in Python bindings
+     * \since QGIS 3.14
+     */
+    const QgsProjectTimeSettings *timeSettings() const SIP_SKIP;
+
+    /**
+     * Returns the project's time settings, which contains the project's temporal range and other
+     * time based settings.
+     *
+     * \since QGIS 3.14
+     */
+    QgsProjectTimeSettings *timeSettings();
+
+    /**
+     * Returns the project's display settings, which settings and properties relating
+     * to how a QgsProject should display values such as map coordinates and bearings.
+     * \note not available in Python bindings
+     * \since QGIS 3.12
+     */
+    const QgsProjectDisplaySettings *displaySettings() const SIP_SKIP;
+
+    /**
+     * Returns the project's display settings, which settings and properties relating
+     * to how a QgsProject should display values such as map coordinates and bearings.
+     * \since QGIS 3.12
+     */
+    QgsProjectDisplaySettings *displaySettings();
+
     /**
      * Returns pointer to the root (invisible) node of the project's layer tree
      * \since QGIS 2.4
@@ -682,12 +916,27 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     QList<QgsVectorLayer *> avoidIntersectionsLayers() const;
 
     /**
-     * A list of layers with which intersections should be avoided.
+     * Sets the list of layers with which intersections should be avoided.
+     * Only used if the avoid intersection mode is set to advanced.
      *
      * \since QGIS 3.0
      */
     void setAvoidIntersectionsLayers( const QList<QgsVectorLayer *> &layers );
 
+    /**
+     * Sets the avoid intersections mode.
+     *
+     * \since QGIS 3.14
+     */
+    void setAvoidIntersectionsMode( const AvoidIntersectionsMode mode );
+
+    /**
+     * Returns the current avoid intersections mode.
+     *
+     * \since QGIS 3.14
+     */
+    AvoidIntersectionsMode avoidIntersectionsMode() const { return mAvoidIntersectionsMode; }
+
     /**
      * A map of custom project variables.
      * To get all available variables including generated ones
@@ -809,7 +1058,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     /**
      * Returns a list of registered map layers with a specified layer type.
      *
-     * Example:
+     * ### Example
      *
      *     QVector<QgsVectorLayer*> vectorLayers = QgsProject::instance()->layers<QgsVectorLayer*>();
      *
@@ -988,6 +1237,19 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     QgsMapLayer *takeMapLayer( QgsMapLayer *layer ) SIP_TRANSFERBACK;
 
+    /**
+     * Returns the main annotation layer associated with the project.
+     *
+     * This layer is always present in projects, and will always be rendered
+     * above any other map layers during map render jobs.
+     *
+     * It forms the default location to place new annotation items which
+     * should appear above all map layers.
+     *
+     * \since QGIS 3.16
+     */
+    QgsAnnotationLayer *mainAnnotationLayer();
+
     /**
      * Removes all registered layers. If the registry has ownership
      * of any layers these layers will also be deleted.
@@ -1158,9 +1420,9 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * \see mapScales()
      * \see mapScalesChanged()
      *
-     * \since QGIS 3.10
+     * \deprecated Use viewSettings() instead
      */
-    void setMapScales( const QVector<double> &scales );
+    Q_DECL_DEPRECATED void setMapScales( const QVector<double> &scales ) SIP_DEPRECATED;
 
     /**
      * Returns the list of custom project map scales.
@@ -1171,9 +1433,9 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * \see setMapScales()
      * \see mapScalesChanged()
      *
-     * \since QGIS 3.10
+     * \deprecated Use viewSettings() instead
      */
-    QVector<double> mapScales() const;
+    Q_DECL_DEPRECATED QVector<double> mapScales() const SIP_DEPRECATED;
 
     /**
      * Sets whether project mapScales() are \a enabled.
@@ -1181,9 +1443,9 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * \see useProjectScales()
      * \see setMapScales()
      *
-     * \since QGIS 3.10
+     * \deprecated Use viewSettings() instead
      */
-    void setUseProjectScales( bool enabled );
+    Q_DECL_DEPRECATED void setUseProjectScales( bool enabled ) SIP_DEPRECATED;
 
     /**
      * Returns TRUE if project mapScales() are enabled.
@@ -1191,9 +1453,9 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * \see setUseProjectScales()
      * \see mapScales()
      *
-     * \since QGIS 3.10
+     * \deprecated Use viewSettings() instead
      */
-    bool useProjectScales() const;
+    Q_DECL_DEPRECATED bool useProjectScales() const SIP_DEPRECATED;
 
     /**
      * Triggers the collection strings of .qgs to be included in ts file and calls writeTsFile()
@@ -1329,6 +1591,13 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     void snappingConfigChanged( const QgsSnappingConfig &config );
 
+    /**
+     * Emitted whenever the avoid intersections mode has changed.
+     *
+     * \since QGIS 3.14
+     */
+    void avoidIntersectionsModeChanged();
+
     /**
      * Emitted whenever the expression variables stored in the project have been changed.
      * \since QGIS 3.0
@@ -1571,9 +1840,9 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * \see mapScales()
      * \see setMapScales()
      *
-     * \since QGIS 3.10
+     * \deprecated Use viewSettings() instead
      */
-    void mapScalesChanged();
+    Q_DECL_DEPRECATED void mapScalesChanged() SIP_DEPRECATED;
 
   public slots:
 
@@ -1624,6 +1893,22 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     */
     void registerTranslatableObjects( QgsTranslationContext *translationContext );
 
+    /**
+     * Sets the data defined properties used for overrides in user defined server
+     * parameters to \a properties
+     *
+     * \since QGIS 3.14
+     */
+    void setDataDefinedServerProperties( const QgsPropertyCollection &properties );
+
+    /**
+     * Returns the data defined properties used for overrides in user defined server
+     * parameters
+     *
+     * \since QGIS 3.14
+     */
+    QgsPropertyCollection dataDefinedServerProperties() const;
+
   private slots:
     void onMapLayersAdded( const QList<QgsMapLayer *> &layers );
     void onMapLayersRemoved( const QList<QgsMapLayer *> &layers );
@@ -1633,15 +1918,6 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
 
     static QgsProject *sProject;
 
-    /**
-     * Set the current project instance to \a project
-     *
-     * \note this is used mainly by the server, which caches the projects and (potentially) needs to switch the current instance on every request
-     * \see instance()
-     * \note not available in Python bindings
-     * \since QGIS 3.2
-     */
-    static void setInstance( QgsProject *project ) SIP_SKIP;
 
     /**
      * Read map layers from project file.
@@ -1651,7 +1927,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * \param flags optional project reading flags
      * \returns TRUE if function worked; else is FALSE
     */
-    bool _getMapLayers( const QDomDocument &doc, QList<QDomNode> &brokenNodes, QgsProject::ReadFlags flags = nullptr );
+    bool _getMapLayers( const QDomDocument &doc, QList<QDomNode> &brokenNodes, QgsProject::ReadFlags flags = QgsProject::ReadFlags() );
 
     /**
      * Set error message from read/write operation
@@ -1672,29 +1948,29 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      *
      * \note not available in Python bindings
      */
-    bool addLayer( const QDomElement &layerElem, QList<QDomNode> &brokenNodes, QgsReadWriteContext &context, QgsProject::ReadFlags flags = nullptr ) SIP_SKIP;
+    bool addLayer( const QDomElement &layerElem, QList<QDomNode> &brokenNodes, QgsReadWriteContext &context, QgsProject::ReadFlags flags = QgsProject::ReadFlags() ) SIP_SKIP;
 
     /**
      * The optional \a flags argument can be used to control layer reading behavior.
      *
      * \note not available in Python bindings
     */
-    void initializeEmbeddedSubtree( const QString &projectFilePath, QgsLayerTreeGroup *group, QgsProject::ReadFlags flags = nullptr ) SIP_SKIP;
+    void initializeEmbeddedSubtree( const QString &projectFilePath, QgsLayerTreeGroup *group, QgsProject::ReadFlags flags = QgsProject::ReadFlags() ) SIP_SKIP;
 
     /**
      * The optional \a flags argument can be used to control layer reading behavior.
      * \note not available in Python bindings
      */
-    void loadEmbeddedNodes( QgsLayerTreeGroup *group, QgsProject::ReadFlags flags = nullptr ) SIP_SKIP;
+    bool loadEmbeddedNodes( QgsLayerTreeGroup *group, QgsProject::ReadFlags flags = QgsProject::ReadFlags() ) SIP_SKIP;
 
     //! Read .qgs file
-    bool readProjectFile( const QString &filename, QgsProject::ReadFlags flags = nullptr );
+    bool readProjectFile( const QString &filename, QgsProject::ReadFlags flags = QgsProject::ReadFlags() );
 
     //! Write .qgs file
     bool writeProjectFile( const QString &filename );
 
     //! Unzip .qgz file then read embedded .qgs file
-    bool unzip( const QString &filename, QgsProject::ReadFlags flags = nullptr );
+    bool unzip( const QString &filename, QgsProject::ReadFlags flags = QgsProject::ReadFlags() );
 
     //! Zip project
     bool zip( const QString &filename );
@@ -1702,6 +1978,9 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     //! Save auxiliary storage to database
     bool saveAuxiliaryStorage( const QString &filename = QString() );
 
+    //! Returns the property definition used for a data defined server property
+    static QgsPropertiesDefinition &dataDefinedServerPropertyDefinitions();
+
     std::unique_ptr< QgsMapLayerStore > mLayerStore;
 
     QString mErrorMessage;
@@ -1716,6 +1995,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     QHash< QString, QPair< QString, bool> > mEmbeddedLayers;
 
     QgsSnappingConfig mSnappingConfig;
+    AvoidIntersectionsMode mAvoidIntersectionsMode = AvoidIntersectionsMode::AllowIntersections;
 
     QgsRelationManager *mRelationManager = nullptr;
 
@@ -1724,10 +2004,18 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
 
     QgsBookmarkManager *mBookmarkManager = nullptr;
 
+    QgsProjectViewSettings *mViewSettings = nullptr;
+
+    QgsProjectTimeSettings *mTimeSettings = nullptr;
+
+    QgsProjectDisplaySettings *mDisplaySettings = nullptr;
+
     QgsLayerTree *mRootGroup = nullptr;
 
     QgsLayerTreeRegistryBridge *mLayerTreeRegistryBridge = nullptr;
 
+    QgsAnnotationLayer *mMainAnnotationLayer = nullptr;
+
     //! map of transaction group: QPair( providerKey, connString ) -> transactionGroup
     QMap< QPair< QString, QString>, QgsTransactionGroup *> mTransactionGroups;
 
@@ -1743,6 +2031,13 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
 
     QFile mFile;                 // current physical project file
 
+    QString mOriginalPath;
+
+    QString mSaveUser;              // last saved user.
+    QString mSaveUserFull;          // last saved user full name.
+    QDateTime mSaveDateTime;
+    QgsProjectVersion mSaveVersion;
+
     /**
      * Manual override for project home path - if empty, home path is automatically
      * created based on file name.
@@ -1761,6 +2056,8 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     int mDirtyBlockCount = 0;
     bool mTrustLayerMetadata = false;
 
+    QgsPropertyCollection mDataDefinedServerProperties;
+
     QgsCoordinateTransformContext mTransformContext;
 
     QgsProjectMetadata mMetadata;
@@ -1777,7 +2074,11 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     friend class QgsProviderRegistry;
 
     // Required by QGIS Server for switching the current project instance
-    friend class QgsConfigCache;
+    friend class QgsServer;
+
+    friend class TestQgsProject;
+
+    Q_DISABLE_COPY( QgsProject )
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsProject::ReadFlags )
@@ -1839,8 +2140,8 @@ class CORE_EXPORT QgsProjectDirtyBlocker
 
 /**
  * Returns the version string found in the given DOM document
-   \returns the version string or an empty string if none found
-   \note not available in Python bindings.
+ * \returns the version string or an empty string if none found
+ * \note not available in Python bindings.
  */
 CORE_EXPORT QgsProjectVersion getVersion( QDomDocument const &doc ) SIP_SKIP;
 
