@@ -16,8 +16,7 @@
 #define QGSPROPERTY_H
 
 #include "qgis_core.h"
-#include "qgis.h"
-#include "qgsproperty_p.h"
+#include "qgis_sip.h"
 #include "qgsexpression.h"
 #include "qgsexpressioncontext.h"
 #include "qgscolorramp.h"
@@ -29,8 +28,10 @@
 #include <QDomElement>
 #include <QDomDocument>
 #include <QColor>
+#include <QDateTime>
 
 class QgsPropertyTransformer;
+class QgsPropertyPrivate;
 
 /**
  * \ingroup core
@@ -76,6 +77,7 @@ class CORE_EXPORT QgsPropertyDefinition
       VerticalAnchor, //!< Vertical anchor point
       SvgPath, //!< Path to an SVG file
       Offset, //!< 2D offset
+      DateTime, //!< DateTime value
       Custom = 3000, //!< Custom property types
     };
 
@@ -100,7 +102,7 @@ class CORE_EXPORT QgsPropertyDefinition
       /**
        * Property requires a boolean value. Note that setting DataTypeBoolean as the required type
        * means that the property also accepts string and numeric fields, as those may be convertible
-       * to a boolean value (Eg "1.0" -> true)
+       * to a boolean value (Eg "1.0" -> TRUE)
        */
       DataTypeBoolean,
     };
@@ -193,7 +195,7 @@ class CORE_EXPORT QgsPropertyDefinition
     StandardPropertyTemplate standardTemplate() const { return mStandardType; }
 
     /**
-     * Returns true if the property is of a type which is compatible with property
+     * Returns TRUE if the property is of a type which is compatible with property
      * override assistants.
      */
     bool supportsAssistant() const;
@@ -233,7 +235,7 @@ class CORE_EXPORT QgsProperty
     //! Property types
     enum Type
     {
-      InvalidProperty, //! Invalid (not set) property
+      InvalidProperty, //!< Invalid (not set) property
       StaticProperty, //!< Static property (QgsStaticProperty)
       FieldBasedProperty, //!< Field based property (QgsFieldBasedProperty)
       ExpressionBasedProperty, //!< Expression based property (QgsExpressionBasedProperty)
@@ -244,7 +246,7 @@ class CORE_EXPORT QgsProperty
      */
     QgsProperty();
 
-    virtual ~QgsProperty() = default;
+    virtual ~QgsProperty();
 
     /**
      * Returns a new ExpressionBasedProperty created from the specified expression.
@@ -267,7 +269,7 @@ class CORE_EXPORT QgsProperty
     QgsProperty &operator=( const QgsProperty &other );
 
     /**
-     * Returns true if the property is not an invalid type.
+     * Returns TRUE if the property is not an invalid type.
      */
     operator bool() const;
 
@@ -342,15 +344,25 @@ class CORE_EXPORT QgsProperty
     /**
      * Prepares the property against a specified expression context. Calling prepare before evaluating the
      * property multiple times allows precalculation of expensive setup tasks such as parsing expressions.
-     * Returns true if preparation was successful.
+     * Returns TRUE if preparation was successful.
      */
     bool prepare( const QgsExpressionContext &context = QgsExpressionContext() ) const;
 
     /**
      * Returns the set of any fields referenced by the property for a specified
      * expression context.
+     * \note The optional argument ignoreContext has been added in QGIS 3.14. When set to TRUE,
+     * even fields not set in context's fields() will be reported - this is useful e.g. with vector tiles
+     * where the actual available field names may not be known beforehand.
      */
-    QSet< QString > referencedFields( const QgsExpressionContext &context = QgsExpressionContext() ) const;
+    QSet< QString > referencedFields( const QgsExpressionContext &context = QgsExpressionContext(), bool ignoreContext = false ) const;
+
+    /**
+     * Returns TRUE if the property is set to a linked project color.
+     *
+     * \since QGIS 3.6
+     */
+    bool isProjectColor() const;
 
     /**
      * Calculates the current value of the property, including any transforms which are set for the property
@@ -358,7 +370,7 @@ class CORE_EXPORT QgsProperty
      * in the expression context can be used to alter the calculated value for the property, so that a property
      * is able to respond to the current environment, layers and features within QGIS.
      * \param defaultValue default value to return if the property is not active or cannot be calculated
-     * \param ok if specified, will be set to true if conversion was successful
+     * \param ok if specified, will be set to TRUE if conversion was successful
      * \returns calculated value for property
      * \see valueAsString()
      * \see valueAsColor()
@@ -368,13 +380,30 @@ class CORE_EXPORT QgsProperty
      */
     QVariant value( const QgsExpressionContext &context, const QVariant &defaultValue = QVariant(), bool *ok SIP_OUT = nullptr ) const;
 
+    /**
+     * Calculates the current value of the property and interprets it as a datetime.
+     * \param context QgsExpressionContext to evaluate the property for.
+     * \param defaultDateTime default datetime to return if the property cannot be calculated as a datetime
+     * \param ok if specified, will be set to TRUE if conversion was successful
+     * \returns value parsed to datetime
+     * \see value()
+     * \see valueAsString()
+     * \see valueAsColor()
+     * \see valueAsDouble()
+     * \see valueAsInt()
+     * \see valueAsBool()
+     * \since QGIS 3.14
+     */
+    QDateTime valueAsDateTime( const QgsExpressionContext &context, const QDateTime &defaultDateTime = QDateTime(), bool *ok SIP_OUT = nullptr ) const;
+
     /**
      * Calculates the current value of the property and interprets it as a string.
      * \param context QgsExpressionContext to evaluate the property for.
      * \param defaultString default string to return if the property cannot be calculated as a string
-     * \param ok if specified, will be set to true if conversion was successful
+     * \param ok if specified, will be set to TRUE if conversion was successful
      * \returns value parsed to string
      * \see value()
+     * \see valueAsDateTime()
      * \see valueAsColor()
      * \see valueAsDouble()
      * \see valueAsInt()
@@ -386,9 +415,10 @@ class CORE_EXPORT QgsProperty
      * Calculates the current value of the property and interprets it as a color.
      * \param context QgsExpressionContext to evaluate the property for.
      * \param defaultColor default color to return if the property cannot be calculated as a color
-     * \param ok if specified, will be set to true if conversion was successful
+     * \param ok if specified, will be set to TRUE if conversion was successful
      * \returns value parsed to color
      * \see value()
+     * \see valueAsDateTime()
      * \see valueAsString()
      * \see valueAsDouble()
      * \see valueAsInt()
@@ -400,9 +430,10 @@ class CORE_EXPORT QgsProperty
      * Calculates the current value of the property and interprets it as a double.
      * \param context QgsExpressionContext to evaluate the property for.
      * \param defaultValue default double to return if the property cannot be calculated as a double
-     * \param ok if specified, will be set to true if conversion was successful
+     * \param ok if specified, will be set to TRUE if conversion was successful
      * \returns value parsed to double
      * \see value()
+     * \see valueAsDateTime()
      * \see valueAsString()
      * \see valueAsColor()
      * \see valueAsInt()
@@ -414,9 +445,10 @@ class CORE_EXPORT QgsProperty
      * Calculates the current value of the property and interprets it as an integer.
      * \param context QgsExpressionContext to evaluate the property for.
      * \param defaultValue default integer to return if the property cannot be calculated as an integer
-     * \param ok if specified, will be set to true if conversion was successful
+     * \param ok if specified, will be set to TRUE if conversion was successful
      * \returns value parsed to integer
      * \see value()
+     * \see valueAsDateTime()
      * \see valueAsString()
      * \see valueAsColor()
      * \see valueAsDouble()
@@ -428,9 +460,10 @@ class CORE_EXPORT QgsProperty
      * Calculates the current value of the property and interprets it as an boolean.
      * \param context QgsExpressionContext to evaluate the property for.
      * \param defaultValue default boolean to return if the property cannot be calculated as an boolean
-     * \param ok if specified, will be set to true if conversion was successful
+     * \param ok if specified, will be set to TRUE if conversion was successful
      * \returns value parsed to boolean
      * \see value()
+     * \see valueAsDateTime()
      * \see valueAsString()
      * \see valueAsColor()
      * \see valueAsDouble()
@@ -457,7 +490,7 @@ class CORE_EXPORT QgsProperty
     /**
      * Sets an optional transformer to use for manipulating the calculated values for the property.
      * \param transformer transformer to install. Ownership is transferred to the property, and any
-     * existing transformer will be deleted. Set to null to remove an existing transformer.
+     * existing transformer will be deleted. Set to NULLPTR to remove an existing transformer.
      * \see transformer()
      */
     void setTransformer( QgsPropertyTransformer *transformer SIP_TRANSFER );
@@ -470,7 +503,7 @@ class CORE_EXPORT QgsProperty
 
     /**
      * Attempts to convert an existing expression based property to a base expression with
-     * corresponding transformer. Returns true if conversion was successful. Note that
+     * corresponding transformer. Returns TRUE if conversion was successful. Note that
      * calling this method requires multiple parsing of expressions, so it should only
      * be called in non-performance critical code.
      */
@@ -482,6 +515,41 @@ class CORE_EXPORT QgsProperty
       return QVariant::fromValue( *this );
     }
 
+
+#ifdef SIP_RUN
+    SIP_PYOBJECT __repr__();
+    % MethodCode
+    QString typeString;
+    QString definitionString;
+    switch ( sipCpp->propertyType() )
+    {
+      case QgsProperty::StaticProperty:
+        typeString = QStringLiteral( "static" );
+        definitionString = sipCpp->staticValue().toString();
+        break;
+
+      case QgsProperty::FieldBasedProperty:
+        typeString = QStringLiteral( "field" );
+        definitionString = sipCpp->field();
+        break;
+
+      case QgsProperty::ExpressionBasedProperty:
+        typeString = QStringLiteral( "expression" );
+        definitionString = sipCpp->expressionString();
+        break;
+
+      case QgsProperty::InvalidProperty:
+        typeString = QStringLiteral( "invalid" );
+        break;
+    }
+
+    QString str = QStringLiteral( "<QgsProperty: %1%2%3>" ).arg( !sipCpp->isActive() && sipCpp->propertyType() != QgsProperty::InvalidProperty ? QStringLiteral( "INACTIVE " ) : QString(),
+                  typeString,
+                  definitionString.isEmpty() ? QString() : QStringLiteral( " (%1)" ).arg( definitionString ) );
+    sipRes = PyUnicode_FromString( str.toUtf8().constData() );
+    % End
+#endif
+
   private:
 
     mutable QExplicitlySharedDataPointer<QgsPropertyPrivate> d;
