@@ -24,6 +24,7 @@
 #include "qgsfields.h"
 #include "qgsrendercontext.h"
 #include "qgsproperty.h"
+#include "qgssymbollayerreference.h"
 
 class QColor;
 class QImage;
@@ -49,6 +50,7 @@ class QgsCurve;
 class QgsPolygon;
 class QgsExpressionContext;
 class QgsPoint;
+class QgsLegendPatchShape;
 
 typedef QList<QgsSymbolLayer *> QgsSymbolLayerList;
 
@@ -130,16 +132,47 @@ class CORE_EXPORT QgsSymbol
      */
     QgsSymbolLayerList symbolLayers() { return mLayers; }
 
+#ifndef SIP_RUN
+
     /**
-     * Returns a specific symbol layer contained in the symbol.
-     * \param layer layer number
-     * \returns corresponding symbol layer
+     * Returns the symbol layer at the specified index
      * \see symbolLayers
      * \see symbolLayerCount
      * \since QGIS 2.7
      */
     QgsSymbolLayer *symbolLayer( int layer );
 
+    /**
+     * Returns the symbol layer at the specified index, const variant
+     * \see symbolLayers
+     * \see symbolLayerCount
+     * \since QGIS 3.12
+     */
+    const QgsSymbolLayer *symbolLayer( int layer ) const;
+#else
+
+    /**
+     * Returns the symbol layer at the specified index. An IndexError will be raised if no layer with the specified index exists.
+     *
+     * \see symbolLayers
+     * \see symbolLayerCount
+     * \since QGIS 2.7
+     */
+    SIP_PYOBJECT symbolLayer( int layer ) SIP_TYPEHINT( QgsSymbolLayer );
+    % MethodCode
+    const int count = sipCpp->symbolLayerCount();
+    if ( a0 < 0 || a0 >= count )
+    {
+      PyErr_SetString( PyExc_IndexError, QByteArray::number( a0 ) );
+      sipIsErr = 1;
+    }
+    else
+    {
+      sipRes = sipConvertFromType( sipCpp->symbolLayer( a0 ), sipType_QgsSymbolLayer, NULL );
+    }
+    % End
+#endif
+
     /**
      * Returns the total number of symbol layers contained in the symbol.
      * \returns count of symbol layers
@@ -149,6 +182,71 @@ class CORE_EXPORT QgsSymbol
      */
     int symbolLayerCount() const { return mLayers.count(); }
 
+#ifdef SIP_RUN
+
+    /**
+     * Returns the number of symbol layers contained in the symbol.
+     */
+    int __len__() const;
+    % MethodCode
+    sipRes = sipCpp->symbolLayerCount();
+    % End
+
+    //! Ensures that bool(obj) returns TRUE (otherwise __len__() would be used)
+    int __bool__() const;
+    % MethodCode
+    sipRes = true;
+    % End
+
+    /**
+    * Returns the symbol layer at the specified ``index``. An IndexError will be raised if no layer with the specified ``index`` exists.
+    *
+    * Indexes can be less than 0, in which case they correspond to layers from the end of the symbol. E.g. an index of -1
+    * corresponds to the last layer in the symbol.
+    *
+    * \since QGIS 3.10
+    */
+    SIP_PYOBJECT __getitem__( int index ) SIP_TYPEHINT( QgsSymbolLayer );
+    % MethodCode
+    const int count = sipCpp->symbolLayerCount();
+    if ( a0 < -count || a0 >= count )
+    {
+      PyErr_SetString( PyExc_IndexError, QByteArray::number( a0 ) );
+      sipIsErr = 1;
+    }
+    else if ( a0 >= 0 )
+    {
+      return sipConvertFromType( sipCpp->symbolLayer( a0 ), sipType_QgsSymbolLayer, NULL );
+    }
+    else
+    {
+      return sipConvertFromType( sipCpp->symbolLayer( count + a0 ), sipType_QgsSymbolLayer, NULL );
+    }
+    % End
+
+    /**
+     * Deletes the layer at the specified ``index``. A layer at the ``index`` must already exist or an IndexError will be raised.
+     *
+     * Indexes can be less than 0, in which case they correspond to layers from the end of the symbol. E.g. an index of -1
+     * corresponds to the last layer in the symbol.
+     *
+     * \since QGIS 3.10
+     */
+    void __delitem__( int index );
+    % MethodCode
+    const int count = sipCpp->symbolLayerCount();
+    if ( a0 >= 0 && a0 < count )
+      sipCpp->deleteSymbolLayer( a0 );
+    else if ( a0 < 0 && a0 >= -count )
+      sipCpp->deleteSymbolLayer( count + a0 );
+    else
+    {
+      PyErr_SetString( PyExc_IndexError, QByteArray::number( a0 ) );
+      sipIsErr = 1;
+    }
+    % End
+#endif
+
     /**
      * Inserts a symbol \a layer to specified \a index.
      * Ownership of \a layer is transferred to the symbol.
@@ -234,12 +332,20 @@ class CORE_EXPORT QgsSymbol
      *
      * Optionally a custom render context may be given in order to ensure that the preview icon exactly
      * matches the settings from that context.
+     * \param painter destination painter
+     * \param size size of the icon
+     * \param customContext the context in which the rendering happens
+     * \param selected set to TRUE to render the symbol in a selected state
+     * \param expressionContext optional custom expression context
+     * \param patchShape optional patch shape to use for symbol preview. If not specified a default shape will be used instead.
      *
      * \see exportImage()
      * \see asImage()
+     * \note Parameter selected added in QGIS 3.10
      * \since QGIS 2.6
      */
-    void drawPreviewIcon( QPainter *painter, QSize size, QgsRenderContext *customContext = nullptr );
+    void drawPreviewIcon( QPainter *painter, QSize size, QgsRenderContext *customContext = nullptr, bool selected = false, const QgsExpressionContext *expressionContext = nullptr,
+                          const QgsLegendPatchShape *patchShape = nullptr );
 
     /**
      * Export the symbol as an image format, to the specified \a path and with the given \a size.
@@ -263,15 +369,28 @@ class CORE_EXPORT QgsSymbol
      */
     QImage asImage( QSize size, QgsRenderContext *customContext = nullptr );
 
+    /**
+     * Flags for controlling how symbol preview images are generated.
+     *
+     * \since QGIS 3.16
+     */
+    enum PreviewFlag
+    {
+      FlagIncludeCrosshairsForMarkerSymbols = 1 << 0, //!< Include a crosshairs reference image in the background of marker symbol previews
+    };
+    Q_DECLARE_FLAGS( PreviewFlags, PreviewFlag )
+
     /**
      * Returns a large (roughly 100x100 pixel) preview image for the symbol.
+     *
      * \param expressionContext optional expression context, for evaluation of
      * data defined symbol properties
+     * \param flags optional flags to control how preview image is generated
      *
      * \see asImage()
      * \see drawPreviewIcon()
      */
-    QImage bigSymbolPreviewImage( QgsExpressionContext *expressionContext = nullptr );
+    QImage bigSymbolPreviewImage( QgsExpressionContext *expressionContext = nullptr, QgsSymbol::PreviewFlags flags = QgsSymbol::FlagIncludeCrosshairsForMarkerSymbols );
 
     /**
      * Returns a string dump of the symbol's properties.
@@ -362,7 +481,7 @@ class CORE_EXPORT QgsSymbol
      * extent. If this option is enabled then features which are partially outside the extent
      * will be clipped. This speeds up rendering of the feature, but may have undesirable
      * side effects for certain symbol types.
-     * \param clipFeaturesToExtent set to true to enable clipping (defaults to TRUE)
+     * \param clipFeaturesToExtent set to TRUE to enable clipping (defaults to TRUE)
      * \see clipFeaturesToExtent
      * \since QGIS 2.9
      */
@@ -483,7 +602,7 @@ class CORE_EXPORT QgsSymbol
      * clockwise for exterior rings and counter-clockwise for interior rings.
      *
      */
-    static void _getPolygon( QPolygonF &pts, QList<QPolygonF> &holes, QgsRenderContext &context, const QgsPolygon &polygon, bool clipToExtent = true, bool correctRingOrientation = false );
+    static void _getPolygon( QPolygonF &pts, QVector<QPolygonF> &holes, QgsRenderContext &context, const QgsPolygon &polygon, bool clipToExtent = true, bool correctRingOrientation = false );
 
     /**
      * Retrieve a cloned list of all layers that make up this symbol.
@@ -514,7 +633,7 @@ class CORE_EXPORT QgsSymbol
     //! Symbol opacity (in the range 0 - 1)
     qreal mOpacity = 1.0;
 
-    RenderHints mRenderHints = nullptr;
+    RenderHints mRenderHints;
     bool mClipFeaturesToExtent = true;
     bool mForceRHR = false;
 
@@ -534,6 +653,32 @@ class CORE_EXPORT QgsSymbol
     //! Initialized in startRender, destroyed in stopRender
     std::unique_ptr< QgsSymbolRenderContext > mSymbolRenderContext;
 
+    /**
+     * Called before symbol layers will be rendered for a particular \a feature.
+     *
+     * This is always followed by a call to stopFeatureRender() after the feature
+     * has been completely rendered (i.e. all parts have been rendered).
+     *
+     * Internally, this notifies all symbol layers which will be used via a call to
+     * QgsSymbolLayer::startFeatureRender().
+     *
+     * \since QGIS 3.12
+     */
+    void startFeatureRender( const QgsFeature &feature, QgsRenderContext &context, int layer = -1 );
+
+    /**
+     * Called after symbol layers have been rendered for a particular \a feature.
+     *
+     * This is always preceded by a call to startFeatureRender() just before the feature
+     * will be rendered.
+     *
+     * Internally, this notifies all symbol layers which were used via a call to
+     * QgsSymbolLayer::stopFeatureRender().
+     *
+     * \since QGIS 3.12
+     */
+    void stopFeatureRender( const QgsFeature &feature, QgsRenderContext &context, int layer = -1 );
+
     Q_DISABLE_COPY( QgsSymbol )
 
 };
@@ -550,7 +695,7 @@ class CORE_EXPORT QgsSymbolRenderContext
 {
   public:
 
-    //TODO QGIS 4.0 - remove mapUnitScale
+    //TODO QGIS 4.0 - remove mapUnitScale and renderunit
 
     /**
      * Constructor for QgsSymbolRenderContext
@@ -563,7 +708,9 @@ class CORE_EXPORT QgsSymbolRenderContext
      * \param fields
      * \param mapUnitScale
      */
-    QgsSymbolRenderContext( QgsRenderContext &c, QgsUnitTypes::RenderUnit u, qreal opacity = 1.0, bool selected = false, QgsSymbol::RenderHints renderHints = nullptr, const QgsFeature *f = nullptr, const QgsFields &fields = QgsFields(), const QgsMapUnitScale &mapUnitScale = QgsMapUnitScale() );
+    QgsSymbolRenderContext( QgsRenderContext &c, QgsUnitTypes::RenderUnit u, qreal opacity = 1.0, bool selected = false, QgsSymbol::RenderHints renderHints = QgsSymbol::RenderHints(), const QgsFeature *f = nullptr, const QgsFields &fields = QgsFields(), const QgsMapUnitScale &mapUnitScale = QgsMapUnitScale() );
+
+    ~QgsSymbolRenderContext();
 
     //! QgsSymbolRenderContext cannot be copied.
     QgsSymbolRenderContext( const QgsSymbolRenderContext &rh ) = delete;
@@ -732,6 +879,22 @@ class CORE_EXPORT QgsSymbolRenderContext
      */
     void setExpressionContextScope( QgsExpressionContextScope *contextScope SIP_TRANSFER );
 
+    /**
+     * Returns the symbol patch shape, to use if rendering symbol preview icons.
+     *
+     * \see setPatchShape()
+     * \since QGIS 3.14
+     */
+    const QgsLegendPatchShape *patchShape() const;
+
+    /**
+     * Sets the symbol patch \a shape, to use if rendering symbol preview icons.
+     *
+     * \see patchShape()
+     * \since QGIS 3.14
+     */
+    void setPatchShape( const QgsLegendPatchShape &shape );
+
   private:
 
 #ifdef SIP_RUN
@@ -750,6 +913,7 @@ class CORE_EXPORT QgsSymbolRenderContext
     int mGeometryPartCount;
     int mGeometryPartNum;
     QgsWkbTypes::GeometryType mOriginalGeometryType = QgsWkbTypes::UnknownGeometry;
+    std::unique_ptr< QgsLegendPatchShape > mPatchShape;
 };
 
 
@@ -930,7 +1094,7 @@ class CORE_EXPORT QgsMarkerSymbol : public QgsSymbol
      * in the \a layer argument. A \a layer of -1 indicates that all symbol layers should be
      * rendered.
      *
-     * If \a selected is true then the symbol will be drawn using the "selected feature"
+     * If \a selected is TRUE then the symbol will be drawn using the "selected feature"
      * style and colors instead of the symbol's normal style.
      */
     void renderPoint( QPointF point, const QgsFeature *f, QgsRenderContext &context, int layer = -1, bool selected = false );
@@ -988,6 +1152,14 @@ class CORE_EXPORT QgsLineSymbol : public QgsSymbol
      */
     void setWidth( double width );
 
+    /**
+     * Sets the width units for the whole symbol (including all symbol layers).
+     * \param unit size units
+     * \since QGIS 3.16
+     */
+    void setWidthUnit( QgsUnitTypes::RenderUnit unit );
+
+
     /**
      * Returns the estimated width for the whole symbol, which is the maximum width of
      * all marker symbol layers in the symbol.
@@ -1038,7 +1210,7 @@ class CORE_EXPORT QgsLineSymbol : public QgsSymbol
      * in the \a layer argument. A \a layer of -1 indicates that all symbol layers should be
      * rendered.
      *
-     * If \a selected is true then the symbol will be drawn using the "selected feature"
+     * If \a selected is TRUE then the symbol will be drawn using the "selected feature"
      * style and colors instead of the symbol's normal style.
      */
     void renderPolyline( const QPolygonF &points, const QgsFeature *f, QgsRenderContext &context, int layer = -1, bool selected = false );
@@ -1088,21 +1260,23 @@ class CORE_EXPORT QgsFillSymbol : public QgsSymbol
      * in the \a layer argument. A \a layer of -1 indicates that all symbol layers should be
      * rendered.
      *
-     * If \a selected is true then the symbol will be drawn using the "selected feature"
+     * If \a selected is TRUE then the symbol will be drawn using the "selected feature"
      * style and colors instead of the symbol's normal style.
      */
-    void renderPolygon( const QPolygonF &points, QList<QPolygonF> *rings, const QgsFeature *f, QgsRenderContext &context, int layer = -1, bool selected = false );
+    void renderPolygon( const QPolygonF &points, const QVector<QPolygonF> *rings, const QgsFeature *f, QgsRenderContext &context, int layer = -1, bool selected = false );
 
     QgsFillSymbol *clone() const override SIP_FACTORY;
 
   private:
 
-    void renderPolygonUsingLayer( QgsSymbolLayer *layer, const QPolygonF &points, QList<QPolygonF> *rings, QgsSymbolRenderContext &context );
+    void renderPolygonUsingLayer( QgsSymbolLayer *layer, const QPolygonF &points, const QVector<QPolygonF> *rings, QgsSymbolRenderContext &context );
     //! Calculates the bounds of a polygon including rings
-    QRectF polygonBounds( const QPolygonF &points, const QList<QPolygonF> *rings ) const;
+    QRectF polygonBounds( const QPolygonF &points, const QVector<QPolygonF> *rings ) const;
     //! Translates the rings in a polygon by a set distance
-    QList<QPolygonF> *translateRings( const QList<QPolygonF> *rings, double dx, double dy ) const;
+    QVector<QPolygonF> *translateRings( const QVector<QPolygonF> *rings, double dx, double dy ) const;
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS( QgsSymbol::PreviewFlags )
+
 #endif
 
